import { 
    SET_CART, 
    PUSH_TO_CART, 
    PULL_FROM_CART, 
    SET_CART_TOTAL 
} from '@/constants/mutationTypes'

import { 
    ADD_TO_CART, 
    DECREASE_FROM_CART, 
    REMOVE_FROM_CART, 
    CHANGE_CART_TOTAL, 
    RESET_CART 
} from '@/constants/actionTypes'

const initialState = {
    cartItems: [],
    cartTotal: 0
}

const state = {
    cartItems: initialState.cartItems,
    cartTotal: initialState.cartTotal
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    cartItems(state) {
        return state.cartItems
    },
    /**
     * 
     * @param {*} state 
     */
    cartTotal(state) {
        return state.cartTotal
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [SET_CART] (state, payload) {
        state.cartItems = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [PUSH_TO_CART] (state, payload) {
        let productFound = state.cartItems.find(item => item.id === payload.id)

        if (productFound) {
            productFound.quantity = productFound.quantity + 1
            return
        }

        state.cartItems = [payload, ...state.cartItems]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [PULL_FROM_CART] (state, payload) {
        let productFoundIndex = state.cartItems.findIndex(item => item.id === payload.item.id)

        if (payload.mode === 'partial') {
            state.cartItems[productFoundIndex].quantity = state.cartItems[productFoundIndex].quantity - 1
        } else {
            state.cartItems.splice(productFoundIndex, 1)
        }
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [SET_CART_TOTAL] (state, payload) {
        state.cartTotal = payload
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [ADD_TO_CART] ({ commit }, payload) {
        commit(PUSH_TO_CART, payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [DECREASE_FROM_CART] ({ commit }, payload) {
        commit(PULL_FROM_CART, {
            item: payload,
            mode: 'partial'
        })
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [REMOVE_FROM_CART] ({ commit }, payload) {
        commit(PULL_FROM_CART, {
            item: payload,
            mode: 'full'
        })
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [CHANGE_CART_TOTAL] ({ commit }, payload) {
        commit(SET_CART_TOTAL, payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [RESET_CART] ({ commit }) {
        commit(SET_CART, [])
        commit(SET_CART_TOTAL, 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}