import { 
    SET_CURRENCIES, 
    SET_LOADING_CURRENCIES,
    AUTOSELECT_CURRENCY
} from '@/constants/mutationTypes'

import { 
    INIT_CURRENCIES, 
    RESET_CURRENCY
} from '@/constants/actionTypes'

const initialState = {
    currencies: [],
    loadingCurrencies: false,
    selectedCurrency: null
}

const state = {
    currencies: initialState.currencies,
    loadingCurrencies: !initialState.currencies,
    selectedCurrency: initialState.selectedCurrency  
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    currencies(state) {
        return state.currencies
    },
    /**
     * 
     * @param {*} state 
     */
    loadingCurrencies(state) {
        return state.loadingCurrencies
    },
    /**
     * 
     * @param {*} state 
     */
    selectedCurrency(state) {
        return state.selectedCurrency
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [SET_CURRENCIES] (state, payload) {
        state.currencies = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [SET_LOADING_CURRENCIES] (state, payload) {
        state.loadingCurrencies = !!payload
    },
    /**
     * 
     * @param {*} state 
     */
    [AUTOSELECT_CURRENCY] (state) {
        state.selectedCurrency = state.currencies.find(item => item.base === true)
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [INIT_CURRENCIES] ({ commit }, payload) {
        commit(SET_CURRENCIES, payload)
        commit(AUTOSELECT_CURRENCY)
        commit(SET_LOADING_CURRENCIES)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [RESET_CURRENCY] ({ commit }) {
        commit(SET_LOADING_CURRENCIES, true)
        commit(SET_CURRENCIES, [])
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}